/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.unitesk.requality.core;

import java.util.ArrayList;
import java.util.List;
import java.util.regex.Pattern;

public class Attribute implements Comparable<Attribute> {
    private String key;
    private Object value;
    private AttributeType type;

    public Attribute(AttributeType type, String key, Object value) {
        this.type = type;
        this.key = key;
        this.value = value;
    }

    public String getKey() {
        return key;
    }

    public Object getValue() {
        return value;
    }

    public AttributeType getType() {
        return type;
    }

    @Override
    public String toString() {
        return "key: "+key+" value: "+value + " ("+super.toString()+")";
    }

    @SuppressWarnings("unchecked")
    @Override
    public boolean equals(Object obj) {
        if(super.equals(obj)) return true;
        if (!(obj instanceof Attribute)) {
            return false;
        }
        Attribute attr = (Attribute) obj;
        if(!key.equals(attr.getKey())) {
            return false;
        }
        if(!type.equals(attr.getType())) {
            return false;
        }
        if (type == AttributeType.LIST && value instanceof List && attr.getValue() instanceof List) {
            List<String> list1 = (List<String>)value;
            List<String> list2 = new ArrayList<String>((List<String>)attr.getValue());
            for (String str : list1) {
                if (list2.contains(str)) {
                    list2.remove(str);
                } else {
                    return false;
                }
            }
            if (!list2.isEmpty()) {
                return false;
            }
        } else
        if(!value.equals(attr.getValue())) {
            return false;
        }
        return true;
    }
    static final Pattern valid_inner_name_pattern = Pattern.compile("[A-Za-z0-9а-яА-Я_[$].-]+[A-Za-z0-9а-яА-Я._[$] -]*");
    static final Pattern valid_name_pattern = Pattern.compile("[A-Za-z0-9а-яА-Я.-]+[A-Za-z0-9а-яА-Я._[$] -]*");
    public static boolean checkNameIsValid(String name, boolean inner){
        return inner ? valid_inner_name_pattern.matcher(name).matches() : valid_name_pattern.matcher(name).matches();
    }

    @Override
    public int compareTo(Attribute o) {
        return this.getKey().compareTo(o.getKey());
    }

    @SuppressWarnings("unchecked")
    public static Attribute deepCopy(Attribute attr) {
        Object value = attr.getValue();
        if (attr.getType() == AttributeType.LIST && value instanceof List) {
            value = new ArrayList<String>((List<String>)value);
            attr = new Attribute(AttributeType.LIST, attr.getKey(), value);
        }
        return attr;
    }

}
