package com.unitesk.requality.core;

import java.util.List;
import java.util.Stack;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class CoreUtils {
    public static String getParentQualifiedId(String QId) {
        return QId.substring(0, QId.lastIndexOf("/"));
    }
    public static String splitWords(String s) {
        if (s == null)
            return null;
        for (int i = 1; i < s.length(); i++) //s.length() is variable
            if (s.charAt(i - 1) >= 'a' && s.charAt(i - 1) <= 'z' &&
                s.charAt(i) >= 'A' && s.charAt(i) <= 'Z') {
                s = s.substring(0, i) + " " + s.substring(i);
            }
        return s;
    }

	public static String htmlEscape(String str) {
        if(str == null)
            return null;
        return htmlSpaces(str.replace("&", "&amp;").replace("<", "&lt;")
                .replace(">", "&gt;"));
    }

    public static String htmlSpaces(String str) {
        if(str == null)
            return null;
        return str.replace("  ",(char)160+""+(char)160);
    }
    
    public static String listToString(List<String> list, String separator){
		return listToString(list.toArray(), separator);
	}
    public static String listToString(Object[] list, String separator){
    	return listToString(list, separator, false);
    }
    public static String listToString(Object[] list, String separator, boolean b){
		String r = "";
		String qu = b?"\"":"";
		for(int i = 0;i<list.length;i++)
			if(!list[i].toString().startsWith("\""))
				r+=qu+list[i]+qu+separator;
			else
				r+=list[i]+separator;
		if(r.equals(""))
			return r;
		return r.substring(0, r.length()-separator.length());
	}
    private static Pattern tagFind = Pattern.compile("\\<[/]?([a-zA-Z]+)(?:[^>]*)\\>");
    private static Stack<String> tags = new Stack<String>(), closed = new Stack<String>(), closedNames = new Stack<String>(), tagsBasic = new Stack<String>();
    /**
     * Method to close all opened tags and fix some problems with tags overlap.
     * Function only considering general structure of tags without deep analysis.
     * Single tags needs to be provide in accordance with XHTML style like 
     * "<br />" instead of "<br>". Otherwise closing tag will be added.
     * Example: for string "<b>one<i> more</b><u><img/> <br>" function will return
     * "<b>one<i> more</i></b><u><img/> <br></br></u>".
     * "<br>" is not comply with XHTML standard and therefore it is provided by closing tag.
     * @param xhtml markup with possible unclosed and overlapping tags 
     * @return correct xhtml markup
     */
	public static String htmlEncapsulate(String newDescr) {
        if (newDescr == null)
            return null;
        StringBuilder res = new StringBuilder();
		Matcher m = tagFind.matcher(newDescr);
		tags.clear();
		closed.clear();
		closedNames.clear();
		tagsBasic.clear();
		int beg = 0;
		boolean ignore = false;
		while(m.find()){
			ignore = false;
			String tag = m.group(0), name = m.group(1);
			if(tag.startsWith("</")){
				res.append(newDescr.substring(beg,m.start()));
				beg = m.start();
				if(!tags.isEmpty()){
					if(!tags.contains(name))
						ignore = true;
					else
						while(!tags.empty() && !tags.peek().equals(name)){
							String t = tags.pop();
							res.append("</").append(t).append(">");
							closed.add(tagsBasic.pop());
							closedNames.add(t);
						}
				}
				if(tags.isEmpty() || ignore)
					ignore = true;
				else{
					tags.pop();
					tagsBasic.pop();
				}
			}else
				if(!tag.endsWith("/>")){
					tags.add(name);
					tagsBasic.add(tag);
				}
			if(!ignore)
				res.append(newDescr.substring(beg,m.end()));
			else
				res.append(newDescr.substring(beg,m.start()));
			while(!closed.isEmpty()){
				String t = closed.pop();
				tags.add(closedNames.pop());
				tagsBasic.add(t);
				res.append(t);
			}
			beg = m.end();
		}
		if(beg<newDescr.length())
			res.append(newDescr.substring(beg));
		while(!tags.isEmpty()){
			res.append("</").append(tags.pop()).append(">");
			tagsBasic.pop();
		}
		return res.toString();
		
	}

}
