package com.unitesk.requality.core;

import java.util.HashMap;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;
import java.util.UUID;
import java.util.regex.Pattern;

import com.unitesk.requality.nodetypes.Comment;
import com.unitesk.requality.nodetypes.DocFolder;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.ReportData;
import com.unitesk.requality.nodetypes.ReportFolder;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.nodetypes.TestPurpose;

public class TreeLogic implements ITreeLogic {
    private Map<String, Set<String>> linktedNodesForLink = new HashMap<String, Set<String>>();
    private Map<String, Map<String, Set<String>>> activeLinkForNode = new HashMap<String, Map<String, Set<String>>>();
    private Map<String, Set<String>> childmap = new HashMap<String, Set<String>>();
    private Map<String, Set<String>> parentmap = new HashMap<String, Set<String>>();
    private Set<String> copyable = new TreeSet<String>();
    private Set<String> sibpasteavalible = new TreeSet<String>();
    private Set<String> autorenamable = new TreeSet<String>();
    
    public TreeLogic() {
        Set<String> childs = new TreeSet<String>();
        childs.add(Requirement.TYPE_NAME);
        childs.add(Comment.TYPE_NAME);
        childs.add(Location.TYPE_NAME);
        childs.add(TestPurpose.TYPE_NAME);
        childmap.put(Requirement.TYPE_NAME, childs);
        
        childs = new TreeSet<String>();
        childs.add(Comment.TYPE_NAME);
        childmap.put(TestPurpose.TYPE_NAME, childs);

        childs = new TreeSet<String>();
        childs.add(ReportFolder.TYPE_NAME);
        childs.add(ReportData.TYPE_NAME);
        childmap.put(ReportFolder.TYPE_NAME, childs);

        childs = new TreeSet<String>();
        childmap.put(Comment.TYPE_NAME, childs);
        
        childs = new TreeSet<String>();
        childmap.put(ReportData.TYPE_NAME, childs);

        childs = new TreeSet<String>();
        childmap.put(Location.TYPE_NAME, childs);

        childs = new TreeSet<String>();
        childs.add(Location.TYPE_NAME);
        childmap.put(Document.TYPE_NAME, childs);

        childs = new TreeSet<String>();
        childs.add(DocFolder.TYPE_NAME);
        childs.add(Document.TYPE_NAME);
        childmap.put(DocFolder.TYPE_NAME, childs);

        childs = new TreeSet<String>();
        childmap.put(Comment.TYPE_NAME, childs);
        
        Set<String> parents = new TreeSet<String>();
        parents.add(Requirement.TYPE_NAME);
        parentmap.put(TestPurpose.TYPE_NAME, parents);

        parents = new TreeSet<String>();
        parents.add(Requirement.TYPE_NAME);
        parentmap.put(Requirement.TYPE_NAME, parents);
        
        parents = new TreeSet<String>();
        parents.add(ReportFolder.TYPE_NAME);
        parentmap.put(ReportFolder.TYPE_NAME, parents);
        
        parents = new TreeSet<String>();
        parents.add(ReportFolder.TYPE_NAME);
        parentmap.put(ReportData.TYPE_NAME, parents);
        
        parents = new TreeSet<String>();
        parents.add(Document.TYPE_NAME);
        parents.add(Requirement.TYPE_NAME);
        parentmap.put(Location.TYPE_NAME, parents);

        parents = new TreeSet<String>();
        parents.add(DocFolder.TYPE_NAME);
        parentmap.put(Document.TYPE_NAME, parents);
        
        parents = new TreeSet<String>();
        parents.add(DocFolder.TYPE_NAME);
        parentmap.put(DocFolder.TYPE_NAME, parents);
        
        parents = new TreeSet<String>();
        parents.add(Requirement.TYPE_NAME);
        parents.add(TestPurpose.TYPE_NAME);
        parentmap.put(Comment.TYPE_NAME, parents);
        
        copyable.add(Comment.TYPE_NAME);
        copyable.add(ReportData.TYPE_NAME);
        copyable.add(Requirement.TYPE_NAME);
        copyable.add(TestPurpose.TYPE_NAME);
        
        sibpasteavalible.add(Comment.TYPE_NAME);
        sibpasteavalible.add(TestPurpose.TYPE_NAME);
        
        autorenamable.add("Comment");
        
        //TODO moveto testdb
        childs = new TreeSet<String>();
        childs.add("TestStep");
        childmap.put("TestProcedure", childs);
        
        childs = new TreeSet<String>();
        childs.add("TestProcedureFolder");
        childs.add("TestProcedure");
        childmap.put("TestProcedureFolder", childs);
        
        childs = new TreeSet<String>();
        childmap.put("TestStep", childs);
        
        parents = new TreeSet<String>();
        parents.add("TestProcedureFolder");
        parentmap.put("TestProcedure", parents);
        
        parents = new TreeSet<String>();
        parents.add("TestProcedureFolder");
        parentmap.put("TestProcedureFolder", parents);
        
        parents = new TreeSet<String>();
        parents.add("TestProcedure");
        parentmap.put("TestStep", parents);
        
        copyable.add("TestStep");
        copyable.add("TestProcedure");
        copyable.add("TestProcedureFolder");
        copyable.add("TODOitem");
        
        sibpasteavalible.add("TestStep");
        
        autorenamable.add("TestStep");
    }
    
    private boolean specialRequirementChildCheck(TreeNode req, String child)
            throws PastingIntoLeafException{
        if(req.getType().equals(Requirement.TYPE_NAME)){
            int size = req.sizeChildren();
            boolean havereqs = false, havetps = false;
            for(int i = 0; i < size; i++){
                String type = req.getChild(i).getType();
                if(type.equals(Requirement.TYPE_NAME))
                    havereqs = true;
                if(type.equals(TestPurpose.TYPE_NAME))
                    havetps = true;
            }
            if(havereqs && child.equals(TestPurpose.TYPE_NAME) 
                    || havetps && child.equals(Requirement.TYPE_NAME)){
                throw new PastingIntoLeafException();
            }
        }
        return true;
    }
    
    @Override
    public boolean canBeParentOfExc(TreeNode what, TreeNode child) throws WrongProjectException, SameIdExistsInChildren, IdEmpty, IdContainsWrongChars{
        if(child == null || what == null)
            return false;
        if (!what.getTreeDB().equals(what.getTreeDB()))
            throw new WrongProjectException();
        return this.canBeParentOf(what.getType(), child) &&
                this.isChildIdValidExc(what, child, child.id);
    }

    @Override
    public boolean canBeParentOf(String whatType, TreeNode child) {
        if(child == null)
            return false;
        Set<String> types = parentmap.get(child.getType());
        if(types == null)
            return true;
        return types.contains(whatType);
    }

    @Override
    public boolean canBeChildOfExc(TreeNode what, TreeNode parent) throws PastingIntoLeafException, WrongProjectException, SameIdExistsInChildren, IdEmpty, IdContainsWrongChars {
        if(parent == null || what == null)
            return false;
        if (!what.getTreeDB().equals(parent.getTreeDB()))
            throw new WrongProjectException();
        return this.canBeChildOfExc(what.getType(), parent) &&
                this.isChildIdValidExc(parent, what, what.id);
    }

    @Override
    public boolean canBeChildOfExc(String whatType, TreeNode parent) 
            throws PastingIntoLeafException{
        if(parent == null)
            return false;
        if(!specialRequirementChildCheck(parent, whatType))
            return false;
        Set<String> types = childmap.get(parent.getType());
        if(types == null)
            return true;
        return types.contains(whatType);
    }

    public static boolean isIdValidExc(String newId) throws IdEmpty, IdContainsWrongChars {
        if(newId.length() == 0)
            throw new IdEmpty();
        if(newId.indexOf("/") > -1)
            throw new IdContainsWrongChars('/');
        return isValidFileName(newId) && newId.length() > 0;
    }
    @Override
    public boolean isIdValidExc(TreeNode node, String newId) throws SameIdExistsInChildren, IdEmpty, IdContainsWrongChars {
        return isChildIdValidExc(node.getParent(), node, newId);
    }

    @Override
    public boolean isChildIdValidExc(TreeNode parent, TreeNode node, String id) throws SameIdExistsInChildren, IdEmpty, IdContainsWrongChars {
        if(!isIdValidExc(id))
            return false;
        if(id.trim().length() == 0)
            return false;
        UUID uuid = null;
        if (node != null)
            uuid = node.getUUId();
        for (TreeNode r : parent.getSortedChildren()) 
            if (r.getId().equals(id.trim()) && !r.getUUId().equals(uuid))
                throw new SameIdExistsInChildren();
        return true;
    }
    
    @Override
    public boolean isChildIdValidExc(TreeNode parent, String id) throws SameIdExistsInChildren, IdEmpty, IdContainsWrongChars {
        return isChildIdValidExc(parent ,null, id);
    }
    
    private static Pattern filepattern = Pattern.compile(
            "# Match a valid Windows filename (unspecified file system).          \n" +
            "^                                # Anchor to start of string.        \n" +
            "(?!                              # Assert filename is not: CON, PRN, \n" +
            "  (?:                            # AUX, NUL, COM1, COM2, COM3, COM4, \n" +
            "    CON|PRN|AUX|NUL|             # COM5, COM6, COM7, COM8, COM9,     \n" +
            "    COM[1-9]|LPT[1-9]            # LPT1, LPT2, LPT3, LPT4, LPT5,     \n" +
            "  )                              # LPT6, LPT7, LPT8, and LPT9...     \n" +
            "  (?:\\.[^.]*)?                  # followed by optional extension    \n" +
            "  $                              # and end of string                 \n" +
            ")                                # End negative lookahead assertion. \n" +
            "([^<>:\"/\\\\|?*\\x00-\\x1F\\ \\xA0]  # First char is not a space.  \n" +
            "[^<>:\"/\\\\|?*\\x00-\\x1F]*)?     # Zero or more valid filename chars.\n" +
            "[^<>:\"/\\\\|?*\\x00-\\x1F\\ \\xA0.]  # Last char is not a space or dot.  \n" +
            "$                                # Anchor to end of string.            ", 
            Pattern.CASE_INSENSITIVE | Pattern.UNICODE_CASE | Pattern.COMMENTS);
    public static Boolean isValidFileName(String name){
        return filepattern.matcher(name).matches() && !(name.startsWith(" ") || name.endsWith(" "));
    }
    
    @Override
    public boolean canBeCreatedInExc(String whatType, String newId, TreeNode parent) throws PastingIntoLeafException, SameIdExistsInChildren, IdEmpty, IdContainsWrongChars {
        return canBeCreatedInExc(whatType, parent) && isChildIdValidExc(parent, newId);
    }

    @Override
    public boolean canBeCreatedInExc(String whatType, TreeNode parent) throws PastingIntoLeafException{
        return canBeChildOfExc(whatType, parent);
    }

    @Override
    public boolean canBeCopiedToExc(TreeNode what, TreeNode newParent,
            boolean changeId) throws PastingIntoLeafException, WrongProjectException, SameIdExistsInChildren, IdEmpty, IdContainsWrongChars {
        try{
            if(canBeChildOfExc(what, newParent) 
                && canBeParentOfExc(newParent, what))
             if(changeId){
                 return true;
             }else{
                 return isChildIdValidExc(newParent, what, what.getId());
             }
            return false;
        }
        catch(SameIdExistsInChildren sce){
            if(changeId)
                return true;
            throw sce;
        }
    }

    @Override
    public Set<String> getPossibleLink(String treeNodeType) {
        Set<String> links = new TreeSet<String>();
        Map<String, Set<String>> map = activeLinkForNode.get(treeNodeType);
        if(map == null)
            return links;
        for (Set<String> list : map.values())
            if(list != null)
                links.addAll(list);
        return links;
    }

    @Override
    public Set<String> getPossibleLinkForAttribute(String treeNodeType,
            String treeNodeAttribute) {
        Set<String> links = new TreeSet<String>();
        Map<String, Set<String>> map = activeLinkForNode.get(treeNodeType);
        if(map == null)
            return links;
        if(map.get(treeNodeAttribute) == null)
            return links;
        return map.get(treeNodeAttribute);
    }

    @Override
    public Set<String> getLinkedNodeTypesToLink(String linkToType) {
        return linktedNodesForLink.get(linkToType);
    }

    @Override
    public boolean canBeDeleted(TreeNode node) {
        if(node.getType().equals(Requirement.TYPE_NAME) 
                && node.getQualifiedId().equals("/Requirements"))
            return false;
        if(node.getType().equals(ReportFolder.TYPE_NAME) 
                && node.getQualifiedId().equals("/Reports"))
            return false;
        if(node.getType().equals(DocFolder.TYPE_NAME) 
                && node.getQualifiedId().equals("/Documents"))
            return false;
        return true;
    }

    @Override
    public boolean isCopyable(TreeNode node){
        return copyable.contains(node.getType());
    }

    @Override
    public boolean canBeParentOf(TreeNode what, TreeNode child) {
        try{
            return canBeParentOfExc(what, child);
        }
        catch(TreeLogicException e)
        {}
        return false;
    }

    @Override
    public boolean canBeChildOf(TreeNode what, TreeNode parent) {
        try{
            return canBeChildOfExc(what, parent);
        }
        catch(TreeLogicException e)
        {}
        return false;
    }

    @Override
    public boolean canBeChildOf(String whatType, TreeNode parent) {
        try{
            return canBeChildOfExc(whatType, parent);
        }
        catch(TreeLogicException e)
        {}
        return false;
    }

    @Override
    public boolean isIdValid(TreeNode node, String newId) {
        try{
            return isIdValidExc(node,newId);
        }
        catch(TreeLogicException e)
        {}
        return false;
    }

    @Override
    public boolean isChildIdValid(TreeNode parent, TreeNode node, String id) {
        try{
            return isChildIdValidExc(parent, node, id);
        }
        catch(TreeLogicException e)
        {}
        return false;
    }

    @Override
    public boolean isChildIdValid(TreeNode parent, String id) {
        try{
            return isChildIdValidExc(parent, id);
        }
        catch(TreeLogicException e)
        {}
        return false;
    }

    @Override
    public boolean canBeParentOf(String whatType, String childType) {
        Set<String> types = parentmap.get(childType);
        if(types == null)
            return true;
        return types.contains(whatType);
    }

    @Override
    public boolean canBeChildOf(String whatType, String parentType) {
        Set<String> types = childmap.get(parentType);
        if(types == null)
            return true;
        return types.contains(whatType);
    }
    
    @Override
    public boolean canBeCreatedIn(String whatType, String newId, TreeNode parent) {
        try{
            return canBeCreatedInExc(whatType, newId, parent);
        }
        catch(TreeLogicException e)
        {}
        return false;
    }

    @Override
    public boolean canBeCopiedTo(TreeNode what, TreeNode newParent,
            boolean changeId) {
        try{
            return canBeCopiedToExc(what, newParent, changeId);
        }
        catch(TreeLogicException e)
        {}
        return false;
    }

    @Override
    public boolean canBeCreatedIn(String whatType, TreeNode parent) {
        try{
            return canBeCreatedInExc(whatType, parent);
        }
        catch(TreeLogicException e)
        {}
        return false;
    }

    @Override
    public boolean isAutoRenamable(String type) {
        return autorenamable.contains(type);
    }

    @Override
    public boolean isSiblinkPasteAvalible(String nodeType) {
        return sibpasteavalible.contains(nodeType);
    }
}
