package com.unitesk.requality.eclipse.core;

import java.util.Map;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.IResourceDelta;
import org.eclipse.core.resources.IResourceDeltaVisitor;
import org.eclipse.core.resources.IncrementalProjectBuilder;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;

import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.RequalityPlugin;
import com.unitesk.requality.tools.DeepFirstTreeWalker;

public class EclipseChecker extends IncrementalProjectBuilder {
    public final static String BUILDER_ID = "com.unitesk.requality.checkers";

    private IResourceDeltaVisitor visitor = new IResourceDeltaVisitor() {
        
        @Override
        public boolean visit(IResourceDelta delta) throws CoreException {
            if(delta.getKind() == IResourceDelta.REMOVED) {
                return false;
            }
            if(delta.getResource().getType() != IResource.FILE) {
                return true;
            }
            TreeNode node = TreesTracker.getNodeByResource((IFile) delta.getResource());
            if(node!=null) {
                RequalityPlugin.getCheckers().check(node);
            }
            return false;
        }
    };

    public EclipseChecker() {
    }

    @Override
    protected IProject[] build(int kind, Map<String, String> args,
            IProgressMonitor monitor) throws CoreException {
        if (kind == FULL_BUILD) {
        	IProject proj = getProject();
        	if(proj == null)
        		return null;
        	TreeDB db = TreesTracker.getTracker().getTree(proj);
        	if(db == null)
        		return null;
            TreeNode root = db.getRootNode();
            if(root != null) {
                DeepFirstTreeWalker tree = new DeepFirstTreeWalker(root);
                for (TreeNode node : tree) {
                    RequalityPlugin.getCheckers().check(node);
                }
            }
        } else {
            if (getDelta(getProject()) != null) {
                getDelta(getProject()).accept(visitor, false);
            }
        }
        return null;
    }
}
