package com.unitesk.requality.eclipse.handlers.document;

import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.ui.PlatformUI;

import com.unitesk.requality.core.ITreeLogic.IdContainsWrongChars;
import com.unitesk.requality.core.ITreeLogic.IdEmpty;
import com.unitesk.requality.core.ITreeLogic.SameIdExistsInChildren;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.nodetypes.DocFolder;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.tools.DeepFirstTreeWalker;

public class RenameDocFolderHandler extends AbstractHandler {

    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        List<TreeNode>nodes = RequalityCNF.getSelectedNodes();
        if(nodes == null || (nodes.size() > 1))
            return null;
        final TreeNode node = nodes.get(0);
        if(node == null)
            return null;
        final DocFolder parentFolder = (DocFolder)node.getParent();
        String id = node.getId().trim();
        InputDialog dlg = new InputDialog(PlatformUI.getWorkbench()
                .getActiveWorkbenchWindow().getShell(), "Change name of Document Folder",
                "Enter Document Folder name", id, new IInputValidator() {
                    @Override
                    public String isValid(String newText) {
                        try {
                            if (!node.getTreeDB().getTreeLogic().isChildIdValidExc(parentFolder, node, newText))
                                return "name is not valid";
                        } catch (IdEmpty e) {
                            return "enter Document Folder name";
                        } catch (IdContainsWrongChars e) {
                            return "name contains wrong characters - " + e.getWrongCharacter();
                        } catch (SameIdExistsInChildren e) {
                            return "child with same id already exists";
                        }
                        return null;
                    }
                });
        if (dlg.open() == Dialog.OK) {
            TreeDB db = node.getTreeDB();
            db.startTransaction("Change id of Document Folder");
            String oldQId = node.getQualifiedId();
            node.setId(dlg.getValue().trim());
            updateRequirements(db, oldQId, node.getQualifiedId());
            db.commit();
        }
        return null;
    }
    
    private void updateRequirements(TreeDB db, String oldQId, String newQId) {
        for (TreeNode node : new DeepFirstTreeWalker(db.getNode(Requirement.getTypeRootQId()))) {
            if (node.getType().equals(Requirement.TYPE_NAME)) {
                Requirement req = (Requirement) node;
                List<String> locs = new ArrayList<String>(req.getLocationQIds());
                boolean flag = false;
                for (String loc : locs) {
                    if (loc.startsWith(oldQId)) {
                        flag = true;
                        req.removeLocation(loc);
                        req.addLocation(newQId + loc.substring(oldQId.length()));
                    }
                }
                if (flag) {
                    req.saveAttributes();
                }
            }
        }
    }
    
    @Override
    public boolean isEnabled() {
        List<TreeNode>nodes = RequalityCNF.getSelectedNodes();
        if(nodes == null || (nodes.size() > 1))
            return false;
        TreeNode node = nodes.get(0);
        TreeNode parent = node.getParent();
        TreeDB db = node.getTreeDB();
        if(parent.getUUId().equals(db.getRootNode().getUUId()))
            return false;
        return true;
    }

}
