/*
 * Copyright 2011 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.unitesk.requality.eclipse.handlers.report;

import java.io.File;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.commands.IHandler;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IFolder;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.dialogs.IInputValidator;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.navigator.CommonNavigator;

import com.unitesk.requality.core.Attribute;
import com.unitesk.requality.core.AttributeType;
import com.unitesk.requality.core.CoreUtils;
import com.unitesk.requality.core.ITreeLogic.IdContainsWrongChars;
import com.unitesk.requality.core.ITreeLogic.IdEmpty;
import com.unitesk.requality.core.ITreeLogic.PastingIntoLeafException;
import com.unitesk.requality.core.ITreeLogic.SameIdExistsInChildren;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.core.instancer.TreeInstancer;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.NodeTypeDescriptor;
import com.unitesk.requality.eclipse.RequalityPlugin;
import com.unitesk.requality.eclipse.core.TreesTracker;
import com.unitesk.requality.eclipse.ui.cnf.RequalityCNF;
import com.unitesk.requality.nodetypes.ReportData;
import com.unitesk.requality.nodetypes.ReportFolder;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.report.ReportGenerator;
import com.unitesk.requality.report.Template;
import com.unitesk.requality.report.TemplateFile;
// TODO report undo
public class GenerateReportHandler extends AbstractHandler implements IHandler {
    public static IContainer getReportFolder(TreeNode node) {
        IFile nodeFile = (IFile) TreesTracker.getResourceByNode(node);
        IContainer res = nodeFile.getParent().getFolder(
                Path.fromOSString(node.getId() + "_resources"));
        return res;
    }

    @Override
    public Object execute(ExecutionEvent event) throws ExecutionException {
        List<TreeNode> nodes = RequalityCNF.getSelectedNodes();
        if(nodes == null)
            return null;
        for(TreeNode node:nodes){
            ReportData report = null;
            if (node instanceof ReportData) {
                report = (ReportData) node;
                generateReport(report); 
            } else if (node instanceof Requirement) {
                final Requirement req = (Requirement) node;
                Shell shell = PlatformUI.getWorkbench().getActiveWorkbenchWindow()
                        .getShell();
                IInputValidator validator = new IInputValidator() {
                    @Override
                    public String isValid(String newText) {
                        ReportFolder folder = (ReportFolder) req.getTreeDB().getNode(
                                "/Reports");
                        if(folder.findChild(newText)!=null) {
                            return "Report with this name already exists";
                        }
                        return null;
                    }
                };
                InputDialog dialog = new InputDialog(shell, "Input Report Name",
                        "Input Report Name", req.getId() + "-report", validator);
                if (dialog.open() == Dialog.OK) {
                    ReportFolder folder = (ReportFolder) req.getTreeDB().getNode(
                            "/Reports");
                    try {
                        if(!folder.getTreeDB().getTreeLogic().canBeCreatedInExc(ReportData.TYPE_NAME, dialog.getValue(), folder)){
                            MessageDialog.openError(PlatformUI.getWorkbench()
                                    .getActiveWorkbenchWindow().getShell(), "Add Element",
                                    "Element of type '" + CoreUtils.splitWords(ReportData.TYPE_NAME) 
                                    + "' cannot be created in " + folder.getUserFriendlyName());
                            return null;
                        }
                    } catch (PastingIntoLeafException e) {
                        MessageDialog.openError(PlatformUI.getWorkbench()
                                .getActiveWorkbenchWindow().getShell(), "Add Element",
                                "Can't add "+CoreUtils.splitWords(ReportData.TYPE_NAME)+" to "+CoreUtils.splitWords(folder.getType())+" with "
                                + "Test Purpose child");
                        return null;
                    } catch (SameIdExistsInChildren e) {
                        MessageDialog.openError(PlatformUI.getWorkbench()
                                .getActiveWorkbenchWindow().getShell(), "Add Element",
                                "Element with same id exists in " + 
                                folder.getUserFriendlyName());
                        return null;
                    } catch (IdEmpty e) {
                        MessageDialog.openError(PlatformUI.getWorkbench()
                                .getActiveWorkbenchWindow().getShell(), "Add Element",
                                "Element id is empty");
                        return null;
                    } catch (IdContainsWrongChars e) {
                        MessageDialog.openError(PlatformUI.getWorkbench()
                                .getActiveWorkbenchWindow().getShell(), "Add Element",
                                "Element id contains wrong character - " 
                                + e.getWrongCharacter());
                        return null;
                    }
                    folder.getTreeDB().startTransaction("Create Report Data");
                    report = (ReportData) folder.createChild(dialog.getValue(),
                            ReportData.TYPE_NAME);
                    report.putAttribute(new Attribute(AttributeType.STRING, ReportData.ATTR_ROOT_REQUIREMENT, node.getQualifiedId()));
                    report.putAttribute(new Attribute(AttributeType.STRING, ReportData.ATTR_TEMPLATE, Template.DEFAULT_ID));
                    report.saveAttributes();
                    folder.getTreeDB().commit();
                    final ReportData r=report;
                    final IViewPart view = PlatformUI.getWorkbench()
                                .getActiveWorkbenchWindow().getActivePage()
                                .findView(RequalityCNF.ID);
                        if (view instanceof CommonNavigator) {
                                PlatformUI.getWorkbench().getActiveWorkbenchWindow()
                                    .getShell().getDisplay().asyncExec(new Runnable() {
                                        public void run() {
                                            StructuredSelection ss=new StructuredSelection(r);
                                            ((RequalityCNF)view).selectReveal(ss);
                                        }
                                    });
                        }
                    //fix of target requirement for report
                }
            } 
        }
        return null;
    }
    private static Shell getShell(){
    	if(Activator.getDefault() == null)
    		return null;
    	IWorkbench w = Activator.getDefault().getWorkbench();
    	if(w==null)
    		return null;
    	IWorkbenchWindow win = w.getActiveWorkbenchWindow();
    	if(win == null)
    		return null;
    	return win.getShell();
    }
    private static void openInformation(String title, String message){
    	Shell s = getShell();
    	if(s == null)
    		return;
    	MessageDialog.openInformation(
                s,
                title,
                message);
    }
    private static void openError(String title, String message){
    	Shell s = getShell();
    	if(s == null)
    		return;
    	MessageDialog.openError(
                s,
                title,
                message);
    }
    public static String generateReport(ReportData node) {
        TreeDB db = node.getTreeDB();
        TreeInstancer ti = new TreeInstancer(db, node.getData());
        TreeDB instTree = TreeDB.load(ti);
        Map<String, NodeTypeDescriptor> nts = RequalityPlugin
                .getAdditionalTypes();
        for (String name : nts.keySet()) {
            instTree.addNodeType(name, nts.get(name).getClazz());
        }
        try{
        	instTree.getNode(node
                .getStartRequirementQId()); 
        	instTree.updateRequirementsNames();
        }catch (RuntimeException re){
        	openInformation("Report generator error", "Target Requirement does not exist.");
        	return null;
        }
        
        String href = null;
        Template tpl = null;
        try {
            tpl = getTemplate(node.getTemplateId());
            if (tpl == null) {
                if(node.getTemplateId() != null){
                	openError("Report generation",
                        "Report template '" + node.getTemplateId() + "' for this report is not avalible.");
                    return null;
                }
                tpl = Activator.getDefault().getReportTemplates().get(Template.DEFAULT_ID);
            }
            String savePath = node.getSaveDir();
            ReportGenerator.generateReport(
                    (Requirement) instTree.getNode(node
                            .getStartRequirementQId()), tpl,
                    savePath, new HashMap<String, Object>(node.getData()));
            // TODO: index file selection
            href = savePath + File.separator
                    + tpl.getTemplates().get(0).getOutputFileName();

            IProject project = TreesTracker.getTracker().getProject(db);
            IPath path = Path.fromOSString(savePath).makeRelativeTo(
                    project.getLocation());

            IFolder folder = project.getFolder(path);
            folder.refreshLocal(IResource.DEPTH_INFINITE, null);
        } catch (Exception e) {
            e.printStackTrace();
        }
        if(href == null)
            return null;
        // TODO report undo
//        node.getTreeDB().startTransaction("Generate Report");
        if(node.getAttributeValue(ReportData.ATTR_TEMPLATE) == null)
            node.putAttribute(new Attribute(AttributeType.STRING, ReportData.ATTR_TEMPLATE,tpl.getId()));
        node.putAttribute(new Attribute(AttributeType.STRING, ReportData.ATTR_DATE, new Date(
                System.currentTimeMillis()).toString()));
        node.putAttribute(new Attribute(AttributeType.STRING, ReportData.ATTR_LAST_GENERATED,
                href.substring(href.lastIndexOf(java.io.File.separatorChar))));
        node.saveAttributes();
        String s = ti.getErrors();
        if(!s.equals(""))
        	openInformation("Report Generator",
                "Warnings: "+s);
//        node.getTreeDB().commit();
        return href;
    }

    public static Template getTemplate(ReportData report){
    	String templateId = report.getTemplateId();
        Template tpl = null;
        if(templateId == null){
            tpl = Activator.getDefault().getReportTemplates()
                    .get(Template.DEFAULT_ID);
            report.putAttribute(new Attribute(AttributeType.STRING, ReportData.ATTR_TEMPLATE,tpl.getId()));
            report.saveAttributes();
        }else{
            tpl = Activator.getDefault().getReportTemplates()
                .get(templateId);
        }
        return tpl;
    }
    public static Template getTemplate(String tid) {
        return Activator.getDefault().getReportTemplates().get(tid);
    }
//remove old image from report before new generation
	public static void clearOldImages(TreeNode nde) {
		if(!nde.getType().equals(ReportData.TYPE_NAME))
			return;
		ReportData node = (ReportData)nde;
        Template tpl = null;
        try {
            tpl = getTemplate(node.getTemplateId());
            if (tpl == null) {
                if(node.getTemplateId() != null){
                    return;
                }
                tpl = Activator.getDefault().getReportTemplates().values()
                        .iterator().next();
            }
            String savePath = node.getSaveDir();
            for (TemplateFile t : tpl.getTemplates()) {
                File outFile = new File(savePath + File.separator
                        + t.getOutputFileName());
                ReportGenerator.clearOldImages(outFile, savePath + File.separator);
            }
             // TODO: index file selection
        } catch (Exception e) {
            e.printStackTrace();
        }
	}
}
