package com.unitesk.requality.eclipse.ui.editors;

import java.io.File;
import java.io.IOException;
import java.net.URI;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.browser.BrowserFunction;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IEditorDescriptor;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IEditorReference;
import org.eclipse.ui.IEditorSite;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.handlers.IHandlerService;
import org.eclipse.ui.ide.IDE;
import org.eclipse.ui.part.EditorPart;

import com.unitesk.requality.core.INodeChangeListener;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.eclipse.Activator;
import com.unitesk.requality.eclipse.RequalityPlugin;
import com.unitesk.requality.eclipse.core.ITreeStateListener;
import com.unitesk.requality.eclipse.core.TreesTracker;
import com.unitesk.requality.eclipse.editors.NodeEditorInput;
import com.unitesk.requality.eclipse.editors.browser.ReqMarker;
import com.unitesk.requality.eclipse.handlers.copypaste.EclipseCopyHandler;
import com.unitesk.requality.eclipse.handlers.copypaste.EclipsePasteHandler;
import com.unitesk.requality.eclipse.handlers.report.OpenReportHandler;
import com.unitesk.requality.nodetypes.Comment;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.ReportData;
import com.unitesk.requality.nodetypes.TestPurpose;
import com.unitesk.requality.report.Template;
import com.unitesk.requality.tests.TestDialogProvider;

// TODO: refacatoring
public class ReportViewer extends EditorPart {
    public static String ID = "com.unitesk.requality.editors.browser.ReportViewer";
    private ReportData rdata;
    private Browser browser;
    private String href;
    
    ITreeStateListener treeStateListener = new ITreeStateListener() {
        @Override
        public void open(TreeDB db) {
        }

        @Override
        public void delete(IProject proj) {
            if (TreesTracker.getResourceByNode(rdata).getProject().equals(proj)) {
                getSite().getShell().getDisplay().syncExec(new Runnable() {
                    @Override
                    public void run() {
                        getEditorSite().getPage().closeEditor(
                                ReportViewer.this, false);
                    }
                });
            }
        }

        @Override
        public void close(TreeDB db) {
            if (rdata.getTreeDB().equals(db)) {
                getSite().getShell().getDisplay().syncExec(new Runnable() {
                    @Override
                    public void run() {
                        getEditorSite().getPage().closeEditor(
                                ReportViewer.this, false);
                    }
                });
            }
        }
    };

    INodeChangeListener nodeChangeListener = new INodeChangeListener() {
		
		@Override
		public void deleted(TreeNode source) {
			if(source.getUUId().equals(rdata.getUUId()))
				getSite().getShell().getDisplay().syncExec(new Runnable() {
                    @Override
                    public void run() {
                        getEditorSite().getPage().closeEditor(
                                ReportViewer.this, false);
                    }
                });
		}
		
		@Override
		public void attributeChange(TreeNode node, String what) {
			if(what.equals(ReportData.ATTR_TEMPLATE) || 
					what.equals(TreeNode.ATTR_UNDEFINED)){
				if(! rdata.getQualifiedId().equals(node.getQualifiedId())){
	                return;
	            }
	            Template tpl = Activator.getDefault().getReportTemplates()
	                    .get(rdata.getTemplateId());
	            String path = null;
	            if (tpl == null) {
	                // get first
	            	if(rdata.getTemplateId() != null)
	            		MessageDialog.openError(PlatformUI.getWorkbench()
	                        .getActiveWorkbenchWindow().getShell(), "Report generation",
	                        "Report template '" + rdata.getTemplateId() + "' for this report is not avalible.");
	                path = ((ReportData)node).getSaveDir() + java.io.File.separator +
	                    (String)node.getAttributeValue(ReportData.ATTR_LAST_GENERATED);
	            } else {
	                path = tpl.getTemplates().get(0)
	                        .getOutputFileName();
	            }
	            final IResource res = OpenReportHandler.getReportFolder((ReportData) node).getFile(
	                    Path.fromOSString(path));
	            if (res.exists()) {
	                getSite().getShell().getDisplay().syncExec(new Runnable() {
	                    @Override
	                    public void run() {
	                        OpenReportHandler.openReport(rdata);
	                        if(!browser.isDisposed())
	                        	browser.refresh();
	                    }
	                });
	            } else {
	                getSite().getShell().getDisplay().syncExec(new Runnable() {
	                    @Override
	                    public void run() {
	                        getEditorSite().getPage().closeEditor(
	                                ReportViewer.this, false);
	                    }
	                });
	            }
			}
		}
	};
    
    private String reportName;

    public ReportViewer() {
    }

    @Override
    public void doSave(IProgressMonitor monitor) {
    }

    @Override
    public void doSaveAs() {
    }

    @Override
    public void init(IEditorSite site, IEditorInput input)
            throws PartInitException {
        setSite(site);
        setInput(input);
        try {
            rdata = (ReportData) ((NodeEditorInput) input).getStorage()
                    .getNode();
            reportName = rdata.getId();
            Template tpl = Activator.getDefault().getReportTemplates()
                    .get(rdata.getTemplateId());
            String savePath = rdata.getSaveDir();
            if (tpl == null) {
                if(rdata.getTemplateId() != null){
                    String lastGen = (String) rdata.getAttributeValue(ReportData.ATTR_LAST_GENERATED);
                    if(lastGen != null){
                        href = Activator.getUrl(savePath + lastGen).toString();
                        MessageDialog.openError(PlatformUI.getWorkbench()
                                .getActiveWorkbenchWindow().getShell(), "Report generation",
                                "Report template '" + rdata.getTemplateId() + "' for this report is not avalible.");
                    }//else we have new template and it is ridiculous
                }else{
                    tpl = Activator.getDefault().getReportTemplates().get(Template.DEFAULT_ID);
                }
            }
            if (tpl != null)
                href = Activator.getUrl(savePath + File.separator
                        + tpl.getTemplates().get(0).getOutputFileName()).toString();
        } catch (CoreException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    @Override
    public boolean isDirty() {
        return false;
    }

    @Override
    public boolean isSaveAsAllowed() {
        return false;
    }
 
    @Override
    public void createPartControl(Composite parent) {
        setPartName("Report: " + reportName);
        browser = RequalityPlugin.newBrowser(parent, RequalityPlugin.getBrowserType());
        IHandlerService handlerService = (IHandlerService) getSite()
                .getService(IHandlerService.class);
        handlerService
            .activateHandler("org.eclipse.ui.edit.copy", new EclipseCopyHandler());
        handlerService
            .activateHandler("org.eclipse.ui.edit.paste", new EclipsePasteHandler());
        
        if (href != null){
            browser.setUrl(href);
            new BrowserFunction(browser,"callAddTest"){
           	 @Override
                public Object function(Object[] arguments) {
           		 	return TestDialogProvider.fireTestCreation(rdata.getTreeDB(), (String)arguments[0], 
           		 			PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell());
                }
            };
            new BrowserFunction(browser, "openEditor"){
            	@Override
            	public Object function(Object[] arguments) {
            		try {
            			String path = (String)arguments[0];
            			if(path.startsWith("file:")){
            				IPath p = Path.fromPortableString((String)arguments[0]);
            				URI uri = URI.create(path);
            				IEditorDescriptor desc = PlatformUI.getWorkbench().
            			        getEditorRegistry().getDefaultEditor(path);
            				if(desc == null){
            					IEditorDescriptor[] d = PlatformUI.getWorkbench().
                    			        getEditorRegistry().getEditors(path);
            					if(d!=null && d.length>0)
            						desc = d[0];
            				}

            				if(desc == null)
        						desc = PlatformUI.getWorkbench().
                    			        getEditorRegistry().getDefaultEditor("*.txt");
            				if(desc != null)
            					return IDE.openEditor(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage(), uri, desc.getId(), true);
            			}else
            			if(path.startsWith("requality://")){
            				path = path.substring(12);
            				String projname = path.substring(0,path.substring(1).indexOf("/")+1);
            				IProject proj = Activator.getProjectByName(projname);
            				if(proj == null)
            					return null;
            				TreeDB db = TreesTracker.getTracker().getTree(proj);
            				if(db == null)
            					return null;
            				try{
            					TreeNode node = db.getNode(path.substring(projname.length()));
            					if(node == null)
            						return null;
            					return openEditor(node);
            				}
            				catch(Exception e){
            				}
            				return true;
            			}else{
            				TreeNode node = null;
            				try{
            					node = rdata.getTreeDB().getNode(path);
            				}catch(Exception e){
            					return null;
            				}
            				return openEditor(node);
            			}
            			
					} catch (PartInitException e) {
						e.printStackTrace();
					}
            		return null;
            	};
            };
            rdata.addChangeListener(nodeChangeListener);
            TreesTracker.getTracker().addListener(treeStateListener);
        }
    }

    protected IEditorPart openEditor(TreeNode node) {
    	String type = node.getType();
    	NodeEditorInput input = new NodeEditorInput(node);
    	if(type.equals(TestPurpose.TYPE_NAME)
                ||type.equals(Comment.TYPE_NAME)
                ||type.equals("TestStep")
                ||type.equals(Location.TYPE_NAME))
    		input = new NodeEditorInput(node.getParent(), node);
        String editorid = UniEditor.ID;
        try 
        {
            if(node.getType().equals(Document.TYPE_NAME))
                editorid = ReqMarker.ID;
            IEditorReference eref;
            if(editorid == ReqMarker.ID){
            	input = new NodeEditorInput(node);
            	eref = RequalityPlugin.findMarkEditorForDocument(
            			node.getQualifiedId(), TreesTracker.getResourceByNode(node).getProject());
            }else
            	eref = RequalityPlugin.findJSEditorForNode(
            			node.getQualifiedId(), TreesTracker.getResourceByNode(node).getProject());
            IEditorPart part;
            if (eref != null)
                part = eref.getEditor(true);
            else{
            	IWorkbenchPage page = PlatformUI.getWorkbench()
                        .getActiveWorkbenchWindow().getActivePage();
                part = IDE.openEditor(page, input,
                         editorid, false);
            }
            if(editorid == UniEditor.ID)
            	((UniEditor)part).setSelection(node);
            part.getSite().getPage().bringToTop(part);
            return part;
            //IDE.openEditor(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage(), input, editorid, false);
        } 
        catch(PartInitException e) 
        {
            e.printStackTrace();
        } catch (CoreException e) {
			e.printStackTrace();
		}
		return null;
	}

	@Override
    public void dispose() {
    	rdata.removeChangeListener(nodeChangeListener);
        TreesTracker.getTracker().removeListener(treeStateListener);
        super.dispose();
    }

    @Override
    public void setFocus() {
        browser.setFocus();
    }

}
