package com.unitesk.requality.report;

import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import com.unitesk.requality.core.CoreUtils;
import com.unitesk.requality.core.TreeDB;
import com.unitesk.requality.core.TreeNode;
import com.unitesk.requality.nodetypes.Document;
import com.unitesk.requality.nodetypes.Location;
import com.unitesk.requality.nodetypes.Requirement;
import com.unitesk.requality.tools.DeepFirstTreeWalker;

public class ReportUtils {
	private TreeDB db;
	private TreeNode root;
    public ReportUtils(TreeDB treeDB, TreeNode root) {
    	db = treeDB;
    	this.root = root;
	}

	public static String getLvlTire(int lvl,String s2){
    	StringBuilder sb = new StringBuilder();
    	for(int i=0; i<lvl-1; i++)
    		sb.append(s2);
    	return sb.toString();
    }
    
    public static String htmlEscape(String str) {
    	return CoreUtils.htmlEscape(str);
    }

    public static String htmlSpaces(String str) {
        return CoreUtils.htmlSpaces(str);
    }

    public static String htmlEncapsulate(String str) {
    	if(str == null)
    		return "";
        return CoreUtils.htmlEncapsulate(str);
    }
    public static String replaceSlashN(String str) {
        if(str == null)
            return null;
        return str.replace("\n","<br>");
    }
    public Object getDocumentNodeModel(String id){
    	Document d = (Document) db.getNode(id);
    	if (d == null)
    		return null;
    	return d.getNodeModel();
    }
    public TreeNode[] getDocuments(){
    	return getDocuments(root.getQualifiedId());
    }
    private Map<String,TreeNode[]> docsmap = new HashMap<String,TreeNode[]>();
    public TreeNode[] getDocuments(String qid){
    	if (qid == null) 
    		return db.getNode(Document.getTypeRootQId()).getSortedChildren(Document.TYPE_NAME);
    	if(!docsmap.containsKey(qid)){
        	Set<TreeNode> docs = new HashSet<TreeNode>();
    		try{
        		DeepFirstTreeWalker dftw = new DeepFirstTreeWalker(db.getNode(qid));
        		Iterator<TreeNode> iter = dftw.iterator();
        		while(iter.hasNext()){
        			TreeNode node = iter.next();
        			if(node.getType().equals(Requirement.TYPE_NAME))
        				for(Location loc: ((Requirement)node).getLocations())
        					docs.add(loc.getDocument());
        		}
        		docsmap.put(qid, docs.toArray(new TreeNode[0]));
        	}catch(Exception e){
        		e.printStackTrace();
        	}
    	}
    	if(docsmap.containsKey(qid))
    		return docsmap.get(qid);
    	return null;
    }
}
